"""
Copyright (C) Cross The Road Electronics.  All rights reserved.
License information can be found in CTRE_LICENSE.txt
For support and suggestions contact support@ctr-electronics.com or file
an issue tracker at https://github.com/CrossTheRoadElec/Phoenix-Releases
"""

from phoenix6.phoenix_native import Native
from phoenix6.status_code import StatusCode
from phoenix6.units import *
import ctypes


class MusicTone:
    """
    Plays a single tone at the user specified frequency.
    
    :param audio_frequency:    Sound frequency to play.  A value of zero will
                               silence the device. The effective frequency range is
                               10-20000 Hz.  Any nonzero frequency less than 10 Hz
                               will be capped to 10 Hz.  Any frequency above 20 kHz
                               will be capped to 20 kHz.
    :type audio_frequency: hertz
    """

    def __init__(self, audio_frequency: hertz):
        self._name = "MusicTone"
        """The name of this control request."""
        self.update_freq_hz: hertz = 100
        """
        The period at which this control will update at.
        This is designated in Hertz, with a minimum of 20 Hz
        (every 50 ms) and a maximum of 1000 Hz (every 1 ms).

        If this field is set to 0 Hz, the control request will
        be sent immediately as a one-shot frame. This may be useful
        for advanced applications that require outputs to be
        synchronized with data acquisition. In this case, we
        recommend not exceeding 50 ms between control calls.
        """
        
        self.audio_frequency = audio_frequency
        """
        Sound frequency to play.  A value of zero will silence the device. The effective
        frequency range is 10-20000 Hz.  Any nonzero frequency less than 10 Hz will be
        capped to 10 Hz.  Any frequency above 20 kHz will be capped to 20 kHz.
        
        - Units: Hz
        
        """

    @property
    def name(self) -> str:
        """
        Gets the name of this control request.

        :returns: Name of the control request
        :rtype: str
        """
        return self._name

    def __str__(self) -> str:
        ss = []
        ss.append("Control: MusicTone")
        ss.append("    audio_frequency: " + str(self.audio_frequency) + " Hz")
        return "\n".join(ss)

    def _send_request(self, network: str, device_hash: int) -> StatusCode:
        """
        Sends this request out over CAN bus to the device for
        the device to apply.

        :param network: Network to send request over
        :type network: str
        :param device_hash: Device to send request to
        :type device_hash: int
        :returns: Status of the send operation
        :rtype: StatusCode
        """
        return StatusCode(Native.instance().c_ctre_phoenix6_RequestControlMusicTone(ctypes.c_char_p(bytes(network, 'utf-8')), device_hash, self.update_freq_hz, self.audio_frequency))

    
    def with_audio_frequency(self, new_audio_frequency: hertz) -> 'MusicTone':
        """
        Modifies this Control Request's audio_frequency parameter and returns itself for
        method-chaining and easier to use request API.
    
        Sound frequency to play.  A value of zero will silence the device. The effective
        frequency range is 10-20000 Hz.  Any nonzero frequency less than 10 Hz will be
        capped to 10 Hz.  Any frequency above 20 kHz will be capped to 20 kHz.
        
        - Units: Hz
        
    
        :param new_audio_frequency: Parameter to modify
        :type new_audio_frequency: hertz
        :returns: Itself
        :rtype: MusicTone
        """
        self.audio_frequency = new_audio_frequency
        return self

    def with_update_freq_hz(self, new_update_freq_hz: hertz) -> 'MusicTone':
        """
        Sets the period at which this control will update at.
        This is designated in Hertz, with a minimum of 20 Hz
        (every 50 ms) and a maximum of 1000 Hz (every 1 ms).

        If this field is set to 0 Hz, the control request will
        be sent immediately as a one-shot frame. This may be useful
        for advanced applications that require outputs to be
        synchronized with data acquisition. In this case, we
        recommend not exceeding 50 ms between control calls.

        :param new_update_freq_hz: Parameter to modify
        :type new_update_freq_hz: hertz
        :returns: Itself
        :rtype: MusicTone
        """
        self.update_freq_hz = new_update_freq_hz
        return self

    @property
    def control_info(self) -> dict:
        """
        Gets information about this control request.

        :returns: Dictonary of control parameter names and corresponding applied values
        :rtype: dict
        """
        control_info = {}
        control_info["name"] = self._name
        control_info["audio_frequency"] = self.audio_frequency
        return control_info
